@@ -1,17 +1,8 @@
 #include "spvm_native.h"
+#include "spvm_socket_util.h"
 
 #include <assert.h>
 
-#ifdef _WIN32
-  #include <ws2tcpip.h>
-  #include <io.h>
-#else
-  #include <sys/types.h>
-  #include <sys/socket.h>
-  #include <netinet/in.h>
-  #include <netinet/ip.h>
-#endif
-
 const char* FILE_NAME = "Sys/Socket/Sockaddr/In.c";
 
 int32_t SPVM__Sys__Socket__Sockaddr__In__new(SPVM_ENV* env, SPVM_VALUE* stack) {
