@@ -1,13 +1,10 @@
 #include "spvm_native.h"
+#include "spvm_socket_util.h"
 
 #include <assert.h>
 
 #ifdef _WIN32
-  #include <ws2tcpip.h>
-  #include <io.h>
 #else
-  #include <sys/types.h>
-  #include <sys/socket.h>
   #include <sys/un.h>
 #endif
 
